@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Init;
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.15';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -50,23 +50,35 @@ sub init($$) {
 # Set log level for Lemonldap::NG logs
 sub logLevelInit {
     my ($class) = @_;
-    my $logger = $class->localConfig->{logger} ||= $class->defaultLogger;
+    my $logger = $class->localConfig->{logger} ||=
+      $ENV{LLNG_DEFAULTLOGGER} || $class->defaultLogger;
     eval "require $logger";
     die $@ if ($@);
-    unless (
-        $class->localConfig->{logLevel} =~ /^(debug|info|notice|warn|error)$/ )
+    my $err;
+    unless ( $class->localConfig->{logLevel} =~
+        /^(?:debug|info|notice|warn|error)$/ )
     {
-        print STDERR 'Bad logLevel value \''
+        $err =
+            'Bad logLevel value \''
           . $class->localConfig->{logLevel}
           . "', switching to 'info'\n";
         $class->localConfig->{logLevel} = 'info';
     }
     $class->logger( $logger->new( $class->localConfig ) );
+    $class->logger->error($err) if $err;
     $class->logger->debug("Logger $logger loaded");
     $logger = $class->localConfig->{userLogger} || $logger;
     eval "require $logger";
     die $@ if ($@);
-    $class->userLogger( $logger->new( $class->localConfig, user => 1 ) );
+    require Lemonldap::NG::Common::Logger::_Duplicate;
+    $class->userLogger(
+        Lemonldap::NG::Common::Logger::_Duplicate->new(
+            $class->localConfig,
+            user   => 1,
+            logger => $logger,
+            dup    => $class->logger
+        )
+    );
     $class->logger->debug("User logger $logger loaded");
 }
 
