@@ -7,7 +7,7 @@ use XML::Simple;
 use Lemonldap::NG::Common::UserAgent;
 use URI;
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.15';
 
 # PROPERTIES
 
@@ -26,6 +26,7 @@ has ua => (
 
 has casSrvList => ( is => 'rw', default => sub { {} }, );
 has casAppList => ( is => 'rw', default => sub { {} }, );
+has srvRules   => ( is => 'rw', default => sub { {} }, );
 has spRules    => ( is => 'rw', default => sub { {} }, );
 has spMacros   => ( is => 'rw', default => sub { {} }, );
 
@@ -41,6 +42,19 @@ sub loadSrv {
         return 0;
     }
     $self->casSrvList( $self->conf->{casSrvMetaDataOptions} );
+
+    # Set rule
+    foreach ( keys %{ $self->conf->{casSrvMetaDataOptions} } ) {
+        my $cond = $self->conf->{casSrvMetaDataOptions}->{$_}
+          ->{casSrvMetaDataOptionsResolutionRule};
+        if ( length $cond ) {
+            my $rule_sub =
+              $self->p->buildRule( $cond, "CAS server resolution" );
+            if ($rule_sub) {
+                $self->srvRules->{$_} = $rule_sub;
+            }
+        }
+    }
     return 1;
 }
 
@@ -226,8 +240,10 @@ sub returnCasServiceValidateSuccess {
     }
     if ($proxies) {
         $self->logger->debug("Add proxies $proxies in response");
-        $s .= "\t\t<cas:proxies>\n\t\t\t<cas:proxy>$_</cas:proxy>\n"
-          foreach ( split( /$self->conf->{multiValuesSeparator}/, $proxies ) );
+        $s .= "\t\t<cas:proxies>\n";
+        $s .= "\t\t\t<cas:proxy>$_</cas:proxy>\n"
+          foreach (
+            reverse( split( $self->conf->{multiValuesSeparator}, $proxies ) ) );
         $s .= "\t\t</cas:proxies>\n";
     }
     $s .= "\t</cas:authenticationSuccess>\n</cas:serviceResponse>\n";
@@ -529,22 +545,28 @@ sub getCasApp {
 
     for my $app ( keys %{ $self->casAppList } ) {
 
-        my $candidateUri =
-          URI->new( $self->casAppList->{$app}->{casAppMetaDataOptionsService} );
-        my $candidateHost  = $candidateUri->authority;
-        my $candidateCanon = $candidateUri->canonical;
+        for my $appservice (
+            split(
+                /\s+/, $self->casAppList->{$app}->{casAppMetaDataOptionsService}
+            )
+          )
+        {
+            my $candidateUri   = URI->new($appservice);
+            my $candidateHost  = $candidateUri->authority;
+            my $candidateCanon = $candidateUri->canonical;
 
-        # Try to match prefix, remembering the longest match found
-        if ( index( $uriCanon, $candidateCanon ) == 0 ) {
-            if ( length($longestCandidate) < length($candidateCanon) ) {
-                $longestCandidate = $candidateCanon;
-                $prefixConfKey    = $app;
+            # Try to match prefix, remembering the longest match found
+            if ( index( $uriCanon, $candidateCanon ) == 0 ) {
+                if ( length($longestCandidate) < length($candidateCanon) ) {
+                    $longestCandidate = $candidateCanon;
+                    $prefixConfKey    = $app;
+                }
             }
-        }
 
-        # Try to match host, only if strict matching is disabled
-        unless ( $self->conf->{casStrictMatching} ) {
-            $hostnameConfKey = $app if ( $hostname eq $candidateHost );
+            # Try to match host, only if strict matching is disabled
+            unless ( $self->conf->{casStrictMatching} ) {
+                $hostnameConfKey = $app if ( $hostname eq $candidateHost );
+            }
         }
     }
 
