@@ -6,7 +6,7 @@ use XML::LibXML;
 use XML::LibXSLT;
 use POSIX qw(strftime);
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.0.15';
 
 # Lemonldap::NG::Portal::Main::Plugin provides addAuthRoute() and
 # addUnauthRoute() methods in addition of Lemonldap::NG::Common::Module.
@@ -69,7 +69,7 @@ sub checkForNotifications {
     }
 
     # Transform notifications
-    my $i   = 0;                                # Files count
+    my $i   = 0;    # Files count
     my $now = strftime "%Y-%m-%d", localtime;
 
     foreach my $file ( values %$notifs ) {
@@ -236,9 +236,8 @@ sub getNotifBack {
 
     # Search for Lemonldap::NG cookie (ciphered)
     my $id;
-    unless ( $id = $req->cookies->{ $self->{conf}->{cookieName} } ) {
-        return $self->p->sendError( $req, 'No cookie found', 401 );
-    }
+    return $self->p->sendError( $req, 'No cookie found', 401 )
+      unless ( $id = $req->cookies->{ $self->{conf}->{cookieName} } );
 
     if ( $req->param('cancel') ) {
         $self->logger->debug('Cancel called -> remove ciphered cookie');
@@ -255,12 +254,12 @@ sub getNotifBack {
         return $self->p->do( $req, [] );
     }
 
-    # Look if all notifications have been accepted. If not, redirect to
-    # portal
+    # Look if all notifications have been accepted.
+    # If not, redirect to Portal
 
     # Try to decrypt Lemonldap::NG ciphered cookie
     $id = $self->p->HANDLER->tsv->{cipher}->decrypt($id)
-      or return $self->sendError( $req, 'Unable to decrypt', 500 );
+      or return $self->sendError( $req, 'Unable to decrypt ciphered id', 400 );
 
     # Check that session exists
     $req->userData( $self->p->HANDLER->retrieveSession( $req, $id ) )
@@ -381,7 +380,6 @@ sub getNotifBack {
 
             # One pending notification has been found and not accepted,
             # restart process to display pending notifications
-            # TODO: is it a good idea to launch all 'endAuth' subs ?
             $self->logger->debug(
                 'Pending notification has been found and not accepted');
             return $self->p->do( $req, [ @{ $self->p->endAuth } ] );
