@@ -5,7 +5,7 @@ use Mouse;
 use JSON qw(from_json to_json);
 use Crypt::URandom;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.0.12';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -34,7 +34,7 @@ has cache => (
             }
             else {
                 $_[0]->{p}->logger->error(
-'Local storage not defined, token will be store in global storage'
+'Local storage not defined, token will be stored into global storage'
                 );
                 return undef;
             }
@@ -42,7 +42,9 @@ has cache => (
     },
 );
 
-sub init { 1 }
+sub init {
+    return 1;
+}
 
 sub createToken {
     my ( $self, $infos ) = @_;
@@ -65,7 +67,7 @@ sub createToken {
 
     if ( $self->cache ) {
         my $id =
-          $infos->{_utime} . '_' . unpack( 's', Crypt::URandom::urandom(2) );
+          $infos->{_utime} . '_' . unpack( 'S', Crypt::URandom::urandom(2) );
 
         # Dereference $infos
         my %h = %$infos;
