@@ -2,14 +2,55 @@
 # Display functions for LemonLDAP::NG Portal
 package Lemonldap::NG::Portal::Main::Display;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.16.1';
 
 package Lemonldap::NG::Portal::Main;
 use strict;
 use Mouse;
 use JSON;
+use URI;
 
-has skinRules => ( is => 'rw' );
+has isPP                     => ( is => 'rw' );
+has speChars                 => ( is => 'rw' );
+has skinRules                => ( is => 'rw' );
+has stayConnected            => ( is => 'rw', default => sub { 0 } );
+has requireOldPwd            => ( is => 'rw', default => sub { 1 } );
+has rememberAuthChoice       => ( is => 'rw', default => sub { 0 } );
+has passwordPolicyActivation => ( is => 'rw', default => sub { 0 } );
+has passwordResetUrl => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = $_[0];
+        my $p    = $self->conf->{portal};
+        $p =~ s#/*$##;
+        return $self->conf->{mailUrl} ? $self->conf->{mailUrl} : "$p/resetpwd";
+    }
+);
+has certificateResetUrl => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = $_[0];
+        my $p    = $self->conf->{portal};
+        $p =~ s#/*$##;
+        return $self->conf->{certificateResetByMailURL}
+          ? $self->conf->{certificateResetByMailURL}
+          : "$p/certificateReset";
+    }
+);
+has registerUrl => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = $_[0];
+        my $p    = $self->conf->{portal};
+        $p =~ s#/*$##;
+        return $self->conf->{registerUrl}
+          ? $self->conf->{registerUrl}
+          : "$p/register";
+    }
+);
 
 sub displayInit {
     my ($self) = @_;
@@ -24,10 +65,58 @@ sub displayInit {
             else {
                 $self->logger->error(
                     qq(Skin rule "$skinRule" returns an error: )
-                      . HANDLER->tsv->{jail}->error );
+                      . HANDLER->tsv->{jail}->error
+                      || 'Unable to compile rule' );
             }
         }
     }
+
+    my $rule = HANDLER->buildSub(
+        HANDLER->substitute( $self->conf->{portalRequireOldPassword} ) );
+    unless ($rule) {
+        my $error = HANDLER->tsv->{jail}->error || 'Unable to compile rule';
+        $self->logger->error("Bad requireOldPwd rule: $error");
+    }
+    $self->requireOldPwd($rule);
+
+    $rule =
+      HANDLER->buildSub( HANDLER->substitute( $self->conf->{stayConnected} ) );
+    unless ($rule) {
+        my $error = HANDLER->tsv->{jail}->error || 'Unable to compile rule';
+        $self->logger->error("Bad stayConnected rule: $error");
+    }
+    $self->stayConnected($rule);
+
+    $rule =
+      HANDLER->buildSub(
+        HANDLER->substitute( $self->conf->{passwordPolicyActivation} ) );
+    unless ($rule) {
+        my $error = HANDLER->tsv->{jail}->error || 'Unable to compile rule';
+        $self->logger->error("Bad passwordPolicyActivation rule: $error");
+    }
+    $self->passwordPolicyActivation($rule);
+
+    $rule =
+      HANDLER->buildSub(
+        HANDLER->substitute( $self->conf->{rememberAuthChoiceRule} ) );
+    unless ($rule) {
+        my $error = HANDLER->tsv->{jail}->error || 'Unable to compile rule';
+        $self->logger->error("Bad rememberAuthChoiceRule rule: $error");
+    }
+    $self->rememberAuthChoice($rule);
+
+    my %speChars = $self->conf->{passwordPolicySpecialChar} eq '__ALL__'
+      ? ()    # Dedup
+      : map { ( s/\s+/<space>/r => 1 ) } split '',
+      $self->conf->{passwordPolicySpecialChar};
+    $self->speChars( join q{ }, sort keys %speChars );
+
+    $self->isPP( $self->speChars
+          || $self->conf->{passwordPolicyMinSize}
+          || $self->conf->{passwordPolicyMinLower}
+          || $self->conf->{passwordPolicyMinUpper}
+          || $self->conf->{passwordPolicyMinDigit}
+          || $self->conf->{passwordPolicyMinSpeChar} );
 }
 
 # Call portal process and set template parameters
@@ -46,9 +135,8 @@ sub display {
         $self->logger->debug('Display: notification detected');
         $skinfile       = 'notification';
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
             NOTIFICATION    => $notif,
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
             AUTH_URL        => $req->{data}->{_url},
@@ -68,21 +156,22 @@ sub display {
         $self->logger->debug('Display: confirm detected');
         $skinfile       = 'confirm';
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $req->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
             AUTH_URL        => $req->{data}->{_url},
             MSG             => $req->info,
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
             ACTIVE_TIMER    => $req->data->{activeTimer},
+            FORM_ACTION     => $req->data->{confirmFormAction} || "#",
             FORM_METHOD     => $self->conf->{confirmFormMethod},
             CHOICE_PARAM    => $self->conf->{authChoiceParam},
             CHOICE_VALUE    => $req->data->{_authChoice},
             CHECK_LOGINS    => $self->conf->{portalCheckLogins}
               && $req->data->{login},
-            ASK_LOGINS => $req->param('checkLogins') || 0,
-            CONFIRMKEY => $self->stamp(),
+            ASK_LOGINS        => $req->param('checkLogins')   || 0,
+            ASK_STAYCONNECTED => $req->param('stayconnected') || 0,
+            CONFIRMKEY        => $self->stamp(),
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -97,10 +186,9 @@ sub display {
         $self->logger->debug('Display: IDP choice detected');
         $skinfile       = 'idpchoice';
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $req->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
             AUTH_URL        => $req->{data}->{_url},
             HIDDEN_INPUTS   => $self->buildHiddenForm($req),
             ACTIVE_TIMER    => $req->data->{activeTimer},
@@ -109,9 +197,10 @@ sub display {
             CHOICE_VALUE    => $req->data->{_authChoice},
             CHECK_LOGINS    => $self->conf->{portalCheckLogins}
               && $req->data->{login},
-            ASK_LOGINS => $req->param('checkLogins') || 0,
-            CONFIRMKEY => $self->stamp(),
-            LIST       => $req->data->{list} || [],
+            ASK_LOGINS        => $req->param('checkLogins')   || 0,
+            ASK_STAYCONNECTED => $req->param('stayconnected') || 0,
+            CONFIRMKEY        => $self->stamp(),
+            LIST              => $req->data->{list} || [],
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -122,23 +211,28 @@ sub display {
 
     # 1.3 There is a message to display
     elsif ( my $info = $req->info ) {
+        my $method =
+          $req->data->{infoFormMethod} || $self->conf->{infoFormMethod};
         $self->logger->debug('Display: info detected');
         $self->logger->debug('Hidden values :');
         $self->logger->debug( " $_: " . $req->{portalHiddenFormValues}->{$_} )
           for keys %{ $req->{portalHiddenFormValues} // {} };
         $skinfile       = 'info';
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $self->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
             MSG             => $info,
-            URL             => $req->{urldc},
-            HIDDEN_INPUTS   => $self->buildHiddenForm($req),
-            ACTIVE_TIMER    => $req->data->{activeTimer},
-            FORM_METHOD     => $self->conf->{infoFormMethod},
-            CHOICE_PARAM    => $self->conf->{authChoiceParam},
-            CHOICE_VALUE    => $req->data->{_authChoice},
+            URL           => $req->{urldc} || $self->conf->{portal},  # Fix 2158
+            HIDDEN_INPUTS => $self->buildOutgoingHiddenForm( $req, $method ),
+            ACTIVE_TIMER  => $req->data->{activeTimer},
+            CHOICE_PARAM  => $self->conf->{authChoiceParam},
+            CHOICE_VALUE  => $req->data->{_authChoice},
+            FORM_METHOD   => $method,
+            (
+                  ( not $req->{urldc} ) ? ( SEND_PARAMS => 1 )
+                : ()
+            ),
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -157,10 +251,9 @@ sub display {
         my $id = $req->{sessionInfo}
           ->{ $self->conf->{openIdAttr} || $self->conf->{whatToTrace} };
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $self->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
             PROVIDERURI     => $p,
             MSG             => $req->info(),
             (
@@ -176,12 +269,12 @@ sub display {
 
     # 2.1 Redirection
     elsif ( $req->{error} == PE_REDIRECT ) {
+        my $method = $req->data->{redirectFormMethod} || 'get';
         $skinfile       = "redirect";
         %templateParams = (
-            MAIN_LOGO     => $self->conf->{portalMainLogo},
             URL           => $req->{urldc},
-            HIDDEN_INPUTS => $self->buildHiddenForm($req),
-            FORM_METHOD   => $req->data->{redirectFormMethod} || 'get',
+            HIDDEN_INPUTS => $self->buildOutgoingHiddenForm( $req, $method ),
+            FORM_METHOD   => $method,
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -196,20 +289,39 @@ sub display {
 
         #utf8::decode($auth_user);
         %templateParams = (
-            MAIN_LOGO => $self->conf->{portalMainLogo},
-            LANGS     => $self->conf->{showLanguages},
             AUTH_USER => $req->{sessionInfo}->{ $self->conf->{portalUserAttr} },
             NEWWINDOW => $self->conf->{portalOpenLinkInNewWindow},
-            LOGOUT_URL          => $self->conf->{portal} . "?logout=1",
-            APPSLIST_ORDER      => $req->{sessionInfo}->{'_appsListOrder'},
-            PING                => $self->conf->{portalPingInterval},
-            REQUIRE_OLDPASSWORD => $self->conf->{portalRequireOldPassword},
-            HIDE_OLDPASSWORD    => 0,
-            DISPLAY_PPOLICY     => $self->conf->{portalDisplayPasswordPolicy},
-            PPOLICY_MINSIZE     => $self->conf->{passwordPolicyMinSize},
-            PPOLICY_MINLOWER    => $self->conf->{passwordPolicyMinLower},
-            PPOLICY_MINUPPER    => $self->conf->{passwordPolicyMinUpper},
-            PPOLICY_MINDIGIT    => $self->conf->{passwordPolicyMinDigit},
+            LOGOUT_URL              => $self->conf->{portal} . "?logout=1",
+            APPSLIST_ORDER          => $req->{sessionInfo}->{'_appsListOrder'},
+            PING                    => $self->conf->{portalPingInterval},
+            DONT_STORE_PASSWORD     => $self->conf->{browsersDontStorePassword},
+            HIDE_OLDPASSWORD        => 0,
+            PPOLICY_NOPOLICY        => !$self->isPP(),
+            ENABLE_PASSWORD_DISPLAY =>
+              $self->conf->{portalEnablePasswordDisplay},
+            ENABLE_CHECKHIBP => $self->conf->{checkHIBP},
+            DISPLAY_PPOLICY  => $self->conf->{portalDisplayPasswordPolicy},
+            PPOLICY_MINSIZE  => $self->conf->{passwordPolicyMinSize},
+            PPOLICY_MINLOWER => $self->conf->{passwordPolicyMinLower},
+            PPOLICY_MINUPPER => $self->conf->{passwordPolicyMinUpper},
+            PPOLICY_MINDIGIT => $self->conf->{passwordPolicyMinDigit},
+            (
+                $self->conf->{passwordPolicyMinSpeChar}
+                  && $self->conf->{passwordPolicySpecialChar}
+                ? ( PPOLICY_MINSPECHAR =>
+                      $self->conf->{passwordPolicyMinSpeChar} )
+                : ()
+            ),
+            (
+                $self->conf->{passwordPolicyMinSpeChar} || $self->speChars()
+                ? ( PPOLICY_ALLOWEDSPECHAR => $self->speChars() )
+                : ()
+            ),
+            (
+                $self->requireOldPwd->( $req, $req->userData )
+                ? ( REQUIRE_OLDPASSWORD => 1 )
+                : ()
+            ),
             $self->menu->params($req),
             (
                 $req->data->{customScript}
@@ -219,15 +331,18 @@ sub display {
         );
     }
 
-    elsif ( $req->error == PE_RENEWSESSION ) {
+    # when upgrading session, the administrator can configure LLNG
+    # to ask only for 2FA
+    elsif ( $req->error == PE_UPGRADESESSION ) {
         $skinfile       = 'upgradesession';
         %templateParams = (
-            MAIN_LOGO  => $self->conf->{portalMainLogo},
-            LANGS      => $self->conf->{showLanguages},
-            MSG        => 'askToRenew',
-            CONFIRMKEY => $self->stamp,
-            PORTAL     => $self->conf->{portal},
-            URL        => $req->data->{_url},
+            FORMACTION   => '/upgradesession',
+            MSG          => 'askToUpgrade',
+            PORTALBUTTON => 1,
+            BUTTON       => 'upgradeSession',
+            CONFIRMKEY   => $self->stamp,
+            PORTAL       => $self->conf->{portal},
+            URL          => $req->data->{_url},
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -236,13 +351,33 @@ sub display {
         );
     }
 
-    elsif ( $req->error == PE_MUSTAUTHN ) {
-        $skinfile       = 'updatesession';
+    # renew uses the same plugin as upgrade, but first factor is mandatory
+    elsif ( $req->error == PE_RENEWSESSION ) {
+        $skinfile       = 'upgradesession';
         %templateParams = (
-            MAIN_LOGO  => $self->conf->{portalMainLogo},
-            LANGS      => $self->conf->{showLanguages},
+            FORMACTION   => '/renewsession',
+            MSG          => 'askToRenew',
+            CONFIRMKEY   => $self->stamp,
+            PORTAL       => $self->conf->{portal},
+            PORTALBUTTON => 1,
+            BUTTON       => 'renewSession',
+            URL          => $req->data->{_url},
+            (
+                $req->data->{customScript}
+                ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
+                : ()
+            ),
+        );
+    }
+
+    # Looks a lot like upgradesession, but no portal logo
+    elsif ( $req->error == PE_MUSTAUTHN ) {
+        $skinfile       = 'upgradesession';
+        %templateParams = (
+            FORMACTION => '/renewsession',
             MSG        => 'PE87',
             CONFIRMKEY => $self->stamp,
+            BUTTON     => 'renewSession',
             PORTAL     => $self->conf->{portal},
             URL        => $req->data->{_url},
             (
@@ -265,17 +400,16 @@ sub display {
             and $req->{error} > PE_OK
             and $req->{error} != PE_FIRSTACCESS
             and $req->{error} != PE_BADCREDENTIALS
+            and $req->{error} != PE_PP_CHANGE_AFTER_RESET
             and $req->{error} != PE_PP_PASSWORD_EXPIRED )
-
-       # and ( $req->{error} == PE_TOKENEXPIRED or $req->{error} == PE_NOTOKEN )
       )
     {
         $skinfile       = 'error';
         %templateParams = (
-            MAIN_LOGO       => $self->conf->{portalMainLogo},
-            LANGS           => $self->conf->{showLanguages},
             AUTH_ERROR      => $req->error,
             AUTH_ERROR_TYPE => $req->error_type,
+            AUTH_ERROR_ROLE => $req->error_role,
+            LOCKTIME        => $req->lockTime(),
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -287,32 +421,45 @@ sub display {
     # 3 Authentication has been refused OR first access
     else {
         $skinfile = 'login';
-        my $login = $self->userId($req);
-        if ( $login eq 'anonymous' ) {
-            $login = '';
-        }
-        elsif ( $req->user ) {
-            $login = $req->{user};
-        }
+        my $login = $req->user;
         %templateParams = (
-            MAIN_LOGO             => $self->conf->{portalMainLogo},
-            LANGS                 => $self->conf->{showLanguages},
             AUTH_ERROR            => $req->error,
             AUTH_ERROR_TYPE       => $req->error_type,
+            AUTH_ERROR_ROLE       => $req->error_role,
             AUTH_URL              => $req->{data}->{_url},
             LOGIN                 => $login,
             DONT_STORE_PASSWORD   => $self->conf->{browsersDontStorePassword},
             CHECK_LOGINS          => $self->conf->{portalCheckLogins},
-            ASK_LOGINS            => $req->param('checkLogins') || 0,
+            ASK_LOGINS            => $req->param('checkLogins')   || 0,
+            ASK_STAYCONNECTED     => $req->param('stayconnected') || 0,
             DISPLAY_RESETPASSWORD => $self->conf->{portalDisplayResetPassword},
             DISPLAY_REGISTER      => $self->conf->{portalDisplayRegister},
-            DISPLAY_UPDATECERTIF  => $self->conf->{portalDisplayCertificateResetByMail},
-            MAILCERTIF_URL        => $self->conf->{certificateResetByMailURL},
-            MAIL_URL              => $self->conf->{mailUrl},
-            REGISTER_URL          => $self->conf->{registerUrl},
-            HIDDEN_INPUTS         => $self->buildHiddenForm($req),
-            STAYCONNECTED         => $self->conf->{stayConnected},
-            SPOOFID               => $self->conf->{impersonationRule},
+            DISPLAY_UPDATECERTIF  =>
+              $self->conf->{portalDisplayCertificateResetByMail},
+            MAILCERTIF_URL => $self->certificateResetUrl,
+            MAIL_URL       => $self->passwordResetUrl,
+            REGISTER_URL   => $self->registerUrl,
+            HIDDEN_INPUTS  => $self->buildHiddenForm($req),
+            IMPERSONATION  => $self->conf->{impersonationRule}
+              || $self->conf->{proxyAuthServiceImpersonation},
+            ENABLE_PASSWORD_DISPLAY =>
+              $self->conf->{portalEnablePasswordDisplay},
+            ENABLE_CHECKHIBP => $self->conf->{checkHIBP},
+            (
+                $self->stayConnected->( $req, $req->sessionInfo )
+                ? ( STAYCONNECTED => 1 )
+                : ()
+            ),
+            (
+                $self->rememberAuthChoice->( $req, $req->sessionInfo )
+                ? ( REMEMBERAUTHCHOICE => 1 )
+                : ()
+            ),
+            REMEMBERAUTHCHOICEDEFAULTCHECKED =>
+              $self->conf->{rememberDefaultChecked} // 0,
+            REMEMBERAUTHCHOICECOOKIENAME => $self->conf->{rememberCookieName}
+              // 'llngrememberauthchoice',
+            REMEMBERAUTHCHOICETIMER => $self->conf->{rememberTimer} // 5,
             (
                 $req->data->{customScript}
                 ? ( CUSTOM_SCRIPT => $req->data->{customScript} )
@@ -320,7 +467,30 @@ sub display {
             ),
         );
 
+        # External links
+        if ( $self->conf->{portalDisplayResetPassword} ) {
+            $templateParams{"MAIL_URL_EXTERNAL"} =
+              $self->_isExternalUrl( $self->passwordResetUrl );
+        }
+        if ( $self->conf->{portalDisplayRegister} ) {
+            $templateParams{"REGISTER_URL_EXTERNAL"} =
+              $self->_isExternalUrl( $self->registerUrl );
+        }
+        if ( $self->conf->{portalDisplayCertificateResetByMail} ) {
+            $templateParams{MAILCERTIF_URL_EXTERNAL} =
+              $self->_isExternalUrl( $self->certificateResetUrl );
+        }
+
         # Display captcha if it's enabled
+        if ( $req->captchaHtml ) {
+            %templateParams =
+              ( %templateParams, CAPTCHA_HTML => $req->captchaHtml, );
+        }
+        if ( $req->token ) {
+            %templateParams = ( %templateParams, TOKEN => $req->token, );
+        }
+
+       # DEPRECATED: This is only used for compatibility with existing templates
         if ( $req->captcha ) {
             %templateParams = (
                 %templateParams,
@@ -328,9 +498,6 @@ sub display {
                 CAPTCHA_SIZE => $self->{conf}->{captcha_size} || 6
             );
         }
-        if ( $req->token ) {
-            %templateParams = ( %templateParams, TOKEN => $req->token, );
-        }
 
         # Show password form if password policy error
         if (
@@ -361,20 +528,37 @@ sub display {
                 CHOICE_PARAM          => $self->conf->{authChoiceParam},
                 CHOICE_VALUE          => $req->data->{_authChoice},
                 OLDPASSWORD           => $self->checkXSSAttack( 'oldpassword',
-                    $req->data->{oldpassword} )
-                ? ""
+                    $req->data->{oldpassword} ) ? ''
                 : $req->data->{oldpassword},
-                HIDE_OLDPASSWORD => $self->conf->{hideOldPassword},
+                HIDE_OLDPASSWORD    => $self->conf->{hideOldPassword},
+                DONT_STORE_PASSWORD => $self->conf->{browsersDontStorePassword},
+                PPOLICY_NOPOLICY    => !$self->isPP(),
+                ENABLE_PASSWORD_DISPLAY =>
+                  $self->conf->{portalEnablePasswordDisplay},
+                ENABLE_CHECKHIBP => $self->conf->{checkHIBP},
                 DISPLAY_PPOLICY  => $self->conf->{portalDisplayPasswordPolicy},
                 PPOLICY_MINSIZE  => $self->conf->{passwordPolicyMinSize},
                 PPOLICY_MINLOWER => $self->conf->{passwordPolicyMinLower},
                 PPOLICY_MINUPPER => $self->conf->{passwordPolicyMinUpper},
                 PPOLICY_MINDIGIT => $self->conf->{passwordPolicyMinDigit},
+                (
+                    $self->conf->{passwordPolicyMinSpeChar}
+                      && $self->conf->{passwordPolicySpecialChar}
+                    ? ( PPOLICY_MINSPECHAR =>
+                          $self->conf->{passwordPolicyMinSpeChar} )
+                    : ()
+                ),
+                (
+                    $self->conf->{passwordPolicyMinSpeChar} || $self->speChars()
+                    ? ( PPOLICY_ALLOWEDSPECHAR => $self->speChars() )
+                    : ()
+                ),
             );
         }
 
         # Disable all forms on:
         # * Logout message
+        # * Account lock
         # * Bad URL error
         elsif ($req->{error} == PE_LOGOUT_OK
             or $req->{error} == PE_WAIT
@@ -388,12 +572,25 @@ sub display {
                 DISPLAY_YUBIKEY_FORM  => 0,
                 AUTH_LOOP             => [],
                 MSG                   => $req->info(),
+                LOCKTIME              => $req->lockTime(),
             );
-
         }
 
         # Display authentication form
         else {
+            my $plugin =
+              $self->loadedModules->{
+                "Lemonldap::NG::Portal::Plugins::FindUser"};
+            my ( $fields, $slogin, $mandatory ) = ( [], '', 0 );
+
+            if (   $plugin
+                && $self->conf->{findUser}
+                && $self->conf->{impersonationRule}
+                && $self->conf->{findUserSearchingAttributes} )
+            {
+                $slogin = $req->data->{findUser};
+                ( $fields, $mandatory ) = $plugin->buildForm();
+            }
 
             # Authentication loop
             if ( $self->conf->{authentication} eq 'Choice'
@@ -404,20 +601,24 @@ sub display {
                     AUTH_LOOP            => $authLoop,
                     CHOICE_PARAM         => $self->conf->{authChoiceParam},
                     CHOICE_VALUE         => $req->data->{_authChoice},
+                    DISPLAY_TAB          => scalar( $req->param("tab") ),
                     DISPLAY_FORM         => 0,
                     DISPLAY_OPENID_FORM  => 0,
                     DISPLAY_YUBIKEY_FORM => 0,
+                    DISPLAY_FINDUSER     => scalar @$fields,
+                    MANDATORY            => $mandatory,
+                    FIELDS               => $fields,
+                    SPOOFID              => $slogin
                 );
             }
 
-            # Choose what form to display if not in a loop
+            # Choose which form to display if not in a loop
             else {
-
                 my $displayType =
                   eval { $self->_authentication->getDisplayType($req) }
                   || 'logo';
 
-                $self->logger->debug("Display type $displayType ");
+                $self->logger->debug("Display type $displayType");
 
                 %templateParams = (
                     %templateParams,
@@ -430,20 +631,21 @@ sub display {
                     : 0,
                     DISPLAY_SSL_FORM  => $displayType =~ /sslform/ ? 1 : 0,
                     DISPLAY_GPG_FORM  => $displayType =~ /gpgform/ ? 1 : 0,
-                    DISPLAY_LOGO_FORM => $displayType eq "logo"    ? 1 : 0,
+                    DISPLAY_LOGO_FORM => $displayType eq "logo" ? 1 : 0,
+                    DISPLAY_FINDUSER  => scalar @$fields,
                     module            => $displayType eq "logo"
                     ? $self->getModule( $req, 'auth' )
                     : "",
-                    AUTH_LOOP => [],
+                    AUTH_LOOP  => [],
                     PORTAL_URL =>
                       ( $displayType eq "logo" ? $self->conf->{portal} : 0 ),
-                    MSG => $req->info(),
+                    MSG       => $req->info(),
+                    MANDATORY => $mandatory,
+                    FIELDS    => $fields,
+                    SPOOFID   => $slogin
                 );
-
             }
-
         }
-
     }
 
     if ( $req->data->{waitingMessage} ) {
@@ -480,6 +682,30 @@ sub staticFile {
     ];
 }
 
+sub buildOutgoingHiddenForm {
+    my ( $self, $req, $method ) = @_;
+    my @keys = keys %{ $req->{portalHiddenFormValues} };
+
+    if ( lc $method eq 'get' ) {
+        my $uri          = URI->new( $req->{urldc} );
+        my %query_params = $uri->query_form;
+
+        # Redirection URL contains query string. Before displaying a form,
+        # we must set the query string parameters as form fields so they can
+        # be preserved #2085
+        if (%query_params) {
+            $self->logger->debug(
+"urldc contains query parameters, setting them as hidden form values"
+            );
+            foreach ( keys %query_params ) {
+                $self->setHiddenFormValue( $req, $_, $query_params{$_}, "", 0 );
+            }
+        }
+    }
+
+    return $self->buildHiddenForm($req);
+}
+
 sub buildHiddenForm {
     my ( $self, $req ) = @_;
     my @keys = keys %{ $req->{portalHiddenFormValues} };
@@ -489,7 +715,9 @@ sub buildHiddenForm {
 
         # Check XSS attacks
         next
-          if $self->checkXSSAttack( $_, $req->{portalHiddenFormValues}->{$_} );
+          if (!$req->data->{safeHiddenFormValues}->{$_}
+            && $self->checkXSSAttack( $_, $req->{portalHiddenFormValues}->{$_} )
+          );
 
         # Build hidden input HTML code
         # 'id' is removed to avoid warning with Choice
@@ -503,10 +731,8 @@ sub buildHiddenForm {
 
 # Return skin name
 # @return skin name
-# TODO: create property for skinRule
 sub getSkin {
     my ( $self, $req ) = @_;
-
     my $skin = $self->conf->{portalSkin};
 
     # Fill sessionInfo to eval rule if empty (unauthenticated user)
@@ -514,13 +740,18 @@ sub getSkin {
     $req->{sessionInfo}->{ipAddr} ||= $req->address;
 
     # Load specific skin from skinRules
-    foreach my $rule ( @{ $self->{skinRules} } ) {
+    foreach my $rule ( @{ $self->skinRules } ) {
         if ( $rule->[1]->( $req, $req->sessionInfo ) ) {
-            if ( -d $self->conf->{templateDir} . '/' . $rule->[0] ) {
+            my $directory = $self->conf->{templateDir} . '/' . $rule->[0];
+            if ( -d $directory ) {
                 $skin = $rule->[0];
                 $self->logger->debug("Skin $skin selected from skin rule");
                 last;
             }
+            else {
+                $self->logger->warn( "Skin $rule->[0] was not selected "
+                      . "because $directory does not exist" );
+            }
         }
     }
 
@@ -557,7 +788,21 @@ sub mkSessionArray {
 
     return "" unless ( ref $sessions eq "ARRAY" and @$sessions );
 
-    my @fields = sort keys %{ $self->conf->{sessionDataToRemember} };
+    # Merge user configuration with plugin self-configuration
+    my %rememberedData = %{ $self->pluginSessionDataToRemember };
+    @rememberedData{ keys %{ $self->conf->{sessionDataToRemember} } } =
+      values %{ $self->conf->{sessionDataToRemember} };
+
+    # Delete fields with an empty/undef/__hidden__ column name
+    delete @rememberedData{
+        grep {
+                 ( not $rememberedData{$_} )
+              or ( $rememberedData{$_} eq "__hidden__" )
+        } keys %rememberedData
+    };
+
+    my @fields = sort( keys %rememberedData );
+
     return $self->loadTemplate(
         $req,
         'sessionArray',
@@ -565,19 +810,21 @@ sub mkSessionArray {
             title        => $title,
             displayUser  => $displayUser,
             displayError => $displayError,
-            fields       => [
-                map { { name => $self->conf->{sessionDataToRemember}->{$_} } }
-                  @fields
-            ],
-            sessions => [
+            fields       => [ map { { name => $rememberedData{$_} } } @fields ],
+            sessions     => [
                 map {
                     my $session = $_;
                     {
                         user   => $session->{user},
                         utime  => $session->{_utime},
                         ip     => $session->{ipAddr},
-                        values => [ map { { v => $session->{$_} } } @fields ],
-                        error  => $session->{error},
+                        values => [
+                            map {
+                                { v => $session->{$_}, k => $_, "k_$_" => 1 }
+                            } @fields
+                        ],
+                        error        => $session->{error},
+                        displayUser  => $displayUser,
                         displayError => $displayError,
                     }
                 } @$sessions
@@ -634,7 +881,8 @@ sub mkOidcConsent {
         'oidcConsents',
         params => {
             partners => [
-                map { {
+                map {
+                    {
                         name        => $_,
                         epoch       => $consents->{$_}->{epoch},
                         scope       => $consents->{$_}->{scope},
@@ -647,4 +895,9 @@ sub mkOidcConsent {
     );
 }
 
+sub _isExternalUrl {
+    my ( $self, $url ) = @_;
+    return ( index( $url, $self->conf->{portal} ) < 0 );
+}
+
 1;
