@@ -15,10 +15,9 @@
 #   * GET /session/my/<type>                     : get session data
 #   * GET /session/my/<type>/key                 : get session key
 #   * DELETE /session/my                         : ask for logout
-#   * DELETE /sessions/my                        : ask for global logout
+#   * DELETE /sessions/my                        : ask for global logout (if GlobalLogout plugin is on)
 #
 # - Authentication
-#   * GET /renewcaptcha                          : get token and captcha image
 #   * POST /sessions/<type>/<session-id>?auth    : authenticate with a fixed
 #                                                  sessionId
 #   * Note that the "getCookie" method (authentification via SOAP) exists for
@@ -35,6 +34,11 @@
 #   * GET /error/<lang>/<errNum>                  : get <errNum> message reference and errors file <lang>.json
 #   Return 'en' error file if no <lang> specified
 #
+# - Endpoints for proxy auth/userdb/password
+#   * POST /proxy/getUser                        : get user attributes (restAuthServer)
+#   * POST /proxy/pwdReset                       : reset password (restPasswordServer)
+#   * POST /proxy/pwdConfirm                     : check password (restAuthServer or restPasswordServer)
+#
 # - Authorizations for connected users (always):
 #   * GET /mysession/?whoami                               : get "my" uid
 #   * GET /mysession/?authorizationfor=<base64-encoded-url>: ask if url is
@@ -44,6 +48,8 @@
 #                                                            (restricted)
 #   * DELETE /mysession/<type>/key                         : delete key in data
 #                                                            (restricted)
+#   * GET    /myapplications                               : get my appplications
+#                                                            list
 #
 # There is no conflict with SOAP server, they can be used together
 
@@ -53,11 +59,16 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 use MIME::Base64;
+use Lemonldap::NG::Portal::Main::Constants qw(
+  URIRE
+  PE_OK
+  portalConsts
+  PE_PASSWORD_OK
+);
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.16.1';
 
-extends
-  qw (Lemonldap::NG::Portal::Main::Plugin Lemonldap::NG::Portal::Lib::Captcha);
+extends 'Lemonldap::NG::Portal::Main::Plugin';
 
 has configStorage => (
     is      => 'ro',
@@ -66,39 +77,41 @@ has configStorage => (
         $_[0]->{p}->HANDLER->localConfig->{configStorage};
     }
 );
-
 has exportedAttr => (
-    is      => 'rw',
+    is      => 'ro',
     lazy    => 1,
     default => sub {
+        my %attributes;
         my $conf = $_[0]->{conf};
-        if ( $conf->{exportedAttr} and $conf->{exportedAttr} !~ /^\s*\+/ ) {
-            return [ split /\s+/, $conf->{exportedAttr} ];
-        }
-        else {
-            my @attributes = (
-                'authenticationLevel', 'groups',
-                'ipAddr',              '_startTime',
-                '_utime',              '_lastSeen',
-                '_session_id',         '_session_kind',
+        if ( !$conf->{exportedAttr} || $conf->{exportedAttr} =~ /^\s*\+/ ) {
+            my @attributes = qw(
+              _lastSeen  _session_id  _session_kind _startTime _utime
+              authenticationLevel     groups        ipAddr
             );
             if ( my $exportedAttr = $conf->{exportedAttr} ) {
                 $exportedAttr =~ s/^\s*\+\s+//;
-                @attributes = ( @attributes, split( /\s+/, $exportedAttr ) );
+                @attributes = ( @attributes, split /[,\s]+/, $exportedAttr );
+            }
+            else {
+                @attributes = (
+                    @attributes,
+                    keys %{ $conf->{macros} },
+                    keys %{ $conf->{exportedVars} }
+                );
             }
 
-            # convert @attributes into hash to remove duplicates
-            my %attributes = map( { $_ => 1 } @attributes );
-            %attributes =
-              ( %attributes, %{ $conf->{exportedVars} }, %{ $conf->{macros} },
-              );
-
-            return '[' . join( ',', keys %attributes ) . ']';
+            # Dedup @attributes
+            %attributes = map { $_ => 1 } @attributes;
+            return '[' . join( ',', sort keys %attributes ) . ']';
+        }
+        else {
+            return
+              '['
+              . join( ',', sort split /[,\s]+/, $conf->{exportedAttr} ) . ']';
         }
     }
 );
-has captcha => ( is => 'rw' );
-has ott     => (
+has ott => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
@@ -115,13 +128,6 @@ sub init {
     my ($self)  = @_;
     my @parents = ('Lemonldap::NG::Portal::Main::Plugin');
     my $add     = 0;
-    if (   $self->conf->{captcha_mail_enabled}
-        || $self->conf->{captcha_login_enabled}
-        || $self->conf->{captcha_register_enabled} )
-    {
-        $self->captcha( $self->p->loadModule('::Lib::Captcha') ) or return 0;
-        $self->addUnauthRoute( renewcaptcha => 'sendCaptcha', ['GET'] );
-    }
     if ( $self->conf->{restConfigServer} ) {
         push @parents, 'Lemonldap::NG::Common::Conf::RESTServer';
         $add++;
@@ -183,9 +189,43 @@ sub init {
           )
 
           ->addAuthRoute(
-            sessions => { my => { ':sessionType' => 'removeSessions' } },
+            session => { my => 'removeSession' },
             ['DELETE']
           );
+
+        if ( $self->conf->{globalLogoutRule} ) {
+            $self->addAuthRoute(
+                sessions => { my => 'removeSessions' },
+                ['DELETE']
+            );
+        }
+    }
+
+    if ( $self->conf->{restPasswordServer} ) {
+        $self->addUnauthRoute(
+            proxy => {
+                'pwdReset' => 'pwdReset',
+            },
+            ['POST']
+        );
+    }
+
+    if ( $self->conf->{restAuthServer} or $self->conf->{restPasswordServer} ) {
+        $self->addUnauthRoute(
+            proxy => {
+                'pwdConfirm' => 'pwdConfirm',
+            },
+            ['POST']
+        );
+    }
+
+    if ( $self->conf->{restAuthServer} ) {
+        $self->addUnauthRoute(
+            proxy => {
+                'getUser' => 'getUser',
+            },
+            ['POST']
+        );
     }
 
     # Methods always available
@@ -205,9 +245,13 @@ sub init {
       ->addAuthRoute(
         mysession => { ':sessionType' => 'updateMySession' },
         ['PUT']
-      );
-    extends @parents if ($add);
+      )
+
+      ->addAuthRoute( myapplications => 'myApplications', ['GET'] );
+
+    extends @parents               if ($add);
     $self->setTypes( $self->conf ) if ( $self->conf->{restSessionServer} );
+
     return 1;
 }
 
@@ -223,20 +267,8 @@ sub newSession {
       or return $self->p->sendError( $req, undef, 400 );
     $infos->{_utime} = time();
 
-    my $force = 0;
-    if ( my $s = delete $infos->{__secret} ) {
-        my $t;
-        if ( $t =
-                $self->conf->{cipher}->decrypt($s)
-            and $t <= time + $self->conf->{restClockTolerance}
-            and $t > time - $self->conf->{restClockTolerance} )
-        {
-            $force = 1;
-        }
-        else {
-            $self->userLogger->error('Bad key, force denied');
-        }
-    }
+    my $secret = delete $infos->{__secret};
+    my $force  = $self->_checkSecret($secret);
 
     if ( $req->param('all') and not $id ) {
         return $self->p->sendError( $req,
@@ -266,7 +298,7 @@ sub newSession {
       unless ($session);
 
     $self->logger->debug(
-        "SOAP request create a new session (" . $session->id . ")" );
+        "REST request create a new session (" . $session->id . ")" );
 
     return $self->p->sendJSONresponse( $req,
         { result => 1, session => $session->data } );
@@ -274,34 +306,35 @@ sub newSession {
 
 sub newAuthSession {
     my ( $self, $req, $id ) = @_;
-    my $t;
-    unless ($t = $req->param('secret')
-        and $t = $self->conf->{cipher}->decrypt($t)
-        and $t <= time
-        and $t > time - 30 )
-    {
+
+    # Check secret
+    my $secret = $req->param('secret');
+    unless ( $self->_checkSecret($secret) ) {
         return $self->p->sendError( $req, 'Bad secret', 403 );
     }
+
+    $req->env->{AuthBasic} = 1;
     $req->{id}    = $id;
     $req->{force} = 1;
     $req->user( $req->param('user') );
     $req->data->{password} = $req->param('password');
     $req->steps( [
             @{ $self->p->beforeAuth },
-            qw(getUser extractFormInfo authenticate setAuthSessionInfo),
+            $self->p->authProcess,
             @{ $self->p->betweenAuthAndData },
             $self->p->sessionData,
             @{ $self->p->afterData },
             $self->p->validSession,
-            @{ $self->p->endAuth },
+            @{ $self->p->endAuth }
         ]
     );
     $req->{error} = $self->p->process($req);
     $self->logger->debug(
         "REST authentication result for $req->{user}: code $req->{error}");
 
-    if ( $req->error > 0 ) {
-        return $self->p->sendError( $req, 'Bad credentials', 401 );
+    if ( $req->error != 0 ) {
+        $self->p->deleteSession($req);
+        return $self->p->sendError( $req, 'Unauthorized', 401 );
     }
     return $self->session( $req, $id );
 }
@@ -318,24 +351,12 @@ sub updateSession {
       or return $self->p->sendError( $req, undef, 400 );
 
     # Get secret if given
-    my $force = 0;
-    if ( my $s = delete $infos->{__secret} ) {
-        my $t;
-        if ( $t =
-                $self->conf->{cipher}->decrypt($s)
-            and $t <= time + $self->conf->{restClockTolerance}
-            and $t > time - $self->conf->{restClockTolerance} )
-        {
-            $force = 1;
-        }
-        else {
-            $self->userLogger->error('Bad key, force denied');
-        }
-    }
+    my $secret = delete $infos->{__secret};
+    my $force  = $self->_checkSecret($secret);
 
     # Get session and store info
     my $session = $self->getApacheSession( $mod, $id, $infos, $force )
-      or return $self->p->sendError( $req, 'Session id does not exists', 400 );
+      or return $self->p->sendError( $req, 'Session Id does not exist', 400 );
 
     return $self->p->sendJSONresponse( $req, { result => 1 } );
 }
@@ -348,17 +369,19 @@ sub delSession {
 
     # Get session
     my $session = $self->getApacheSession( $mod, $id )
-      or return $self->p->sendError( $req, 'Session id does not exists', 400 );
+      or return $self->p->sendError( $req, 'Session Id does not exist', 400 );
 
     # Delete it
     $self->logger->debug("REST request to delete session $id");
     my $res = $self->p->_deleteSession( $req, $session );
     $self->logger->debug(" Result is $res");
+
     return $self->p->sendJSONresponse( $req, { result => $res } );
 }
 
 sub delMySession {
     my ( $self, $req, $id ) = @_;
+
     return $self->delSession( $req, $req->userData->{_session_id} );
 }
 
@@ -366,10 +389,9 @@ sub mysession {
     my ( $self, $req ) = @_;
 
     # 1. whoami
-    if ( defined $req->param('whoami') ) {
-        return $self->p->sendJSONresponse( $req,
-            { result => $req->userData->{ $self->conf->{whatToTrace} } } );
-    }
+    return $self->p->sendJSONresponse( $req,
+        { result => $req->userData->{ $self->conf->{whatToTrace} } } )
+      if defined $req->param('whoami');
 
     if ( defined $req->param('gettoken') ) {
         return $self->p->sendJSONresponse( $req,
@@ -389,7 +411,8 @@ sub mysession {
           if ( $self->p->checkXSSAttack( 'authorizationfor', $req->urldc ) );
 
         # Split URL
-        my ( $host, $uri ) = ( $req->urldc =~ m#^https?://([^/]+)(/.*)?$# );
+        $req->urldc =~ URIRE;
+        my ( $host, $uri ) = ( $3 . ( $4 ? ":$4" : '' ), $5 );
         $uri ||= '/';
         return $self->p->sendError( $req, "Bad URL $req->{urldc}", 400 )
           unless ($host);
@@ -402,6 +425,7 @@ sub mysession {
         $self->logger->debug(" Result is $res");
         return $self->p->sendJSONresponse( $req, { result => $res } );
     }
+
     return $self->p->sendError( $req,
         'whoami or authorizationfor is required', 400 );
 }
@@ -409,7 +433,17 @@ sub mysession {
 sub getMyKey {
     my ( $self, $req, $key ) = @_;
     $key ||= '';
-    $self->logger->debug("Request to get personal session info -> Key : $key");
+    if ($key) {
+        $self->logger->debug(
+            "REST request to get personal session attribute: $key");
+    }
+    else {
+        my $keys = $self->exportedAttr;
+        $keys =~ s/\[|\]//g;
+        $keys =~ s/,/, /g;
+        $self->logger->debug("REST request to get exported attributes: $keys");
+    }
+
     return $self->session(
         $req,
         $req->userData->{_session_id},
@@ -421,6 +455,7 @@ sub updateMySession {
     my ( $self, $req ) = @_;
     my $res   = 0;
     my $mKeys = [];
+
     if ( my $token = $req->param('token') ) {
         if ( $self->ott->getToken($token) ) {
             if ( $req->param('sessionType') eq 'persistent' ) {
@@ -455,9 +490,8 @@ sub updateMySession {
     else {
         $self->logger->error('Update session request without token');
     }
-    unless ($res) {
-        return $self->p->sendError( $req, 'Modification refused', 403 );
-    }
+
+    return $self->p->sendError( $req, 'Modification refused', 403 ) unless $res;
     return $self->p->sendJSONresponse( $req,
         { result => 1, count => $res, modifiedKeys => $mKeys } );
 }
@@ -468,6 +502,7 @@ sub delKeyInMySession {
     my $mKeys = [];
     my $dkey  = $req->param('key');
     my $sub   = $req->param('sub');
+
     if ( my $token = $req->param('token') ) {
         if ( $self->ott->getToken($token) ) {
             if ( $req->param('sessionType') eq 'persistent' ) {
@@ -540,9 +575,8 @@ sub delKeyInMySession {
     else {
         $self->logger->error('Update session request without token');
     }
-    unless ($res) {
-        return $self->p->sendError( $req, 'Modification refused', 403 );
-    }
+
+    return $self->p->sendError( $req, 'Modification refused', 403 ) unless $res;
     return $self->p->sendJSONresponse( $req,
         { result => 1, count => $res, modifiedKeys => $dkey } );
 }
@@ -557,16 +591,35 @@ sub getError {
     return $self->p->sendJSONresponse(
         $req,
         {
-            result   => 1,
-            lang     => $lang,
-            errorNum => $errNum ? $errNum : 'all',
+            result        => 1,
+            lang          => $lang,
+            errorNum      => $errNum ? $errNum : 'all',
             errorsFileURL =>
               "$self->{conf}->{staticPrefix}/languages/$lang.json",
-            ( $errNum ? ( errorMsgRef => "PE$errNum" ) : () ),
+            ( $errNum ? ( errorMsgRef => "PE$errNum" ) : () )
         }
     );
 }
 
+sub removeSession {
+    my ( $self, $req ) = @_;
+    my $id = $req->userData->{_session_id};
+    return $self->p->sendError( $req, 'ID is required', 400 ) unless ($id);
+    my $mod = $self->getGlobal()
+      or return $self->p->sendError( $req, undef, 400 );
+
+    # Get session
+    my $session = $self->getApacheSession( $mod, $id )
+      or return $self->p->sendError( $req, 'Session Id does not exist', 400 );
+
+    # Delete it
+    $self->logger->debug("REST request to delete global session $id");
+    my $res = $self->p->_deleteSession( $req, $session );
+    $self->logger->debug(" Result is $res");
+
+    return $self->p->sendJSONresponse( $req, { result => $res } );
+}
+
 sub removeSessions {
     my ( $self, $req ) = @_;
     my $glPlugin =
@@ -577,12 +630,190 @@ sub removeSessions {
     return $self->p->sendJSONresponse( $req, { result => $nbr } );
 }
 
-sub sendCaptcha {
-    my ( $self,  $req )   = @_;
-    my ( $token, $image ) = $self->captcha->getCaptcha($req);
+sub pwdReset {
+    my ( $self, $req ) = @_;
+
+    $self->logger->debug("Entering REST pwdReset method");
+
+    unless ( $self->p->_passwordDB ) {
+        $self->logger->error(
+                "No Password module configured on this server, "
+              . "cannot execute password change" );
+        return $self->p->sendJSONresponse( $req, { 'result' => JSON::false } );
+    }
+
+    my $jsonBody = eval { from_json( $req->content ) };
+    if ($@) {
+        $self->logger->error("Received invalid JSON $@");
+        return $self->p->sendError( $req, "Invalid JSON", 400 );
+    }
+
+    my $user     = $jsonBody->{user};
+    my $mail     = $jsonBody->{mail};
+    my $password = $jsonBody->{password};
+
+    unless ( $user or $mail ) {
+        $self->logger->error("Missing user or mail argument");
+        return $self->p->sendError( $req, "Missing user or mail argument",
+            400 );
+    }
+
+    unless ($password) {
+        $self->logger->error("Missing password argument");
+        return $self->p->sendError( $req, "Missing password argument", 400 );
+    }
+
+    $req->user( $user || $mail );
+    $req->steps( ['getUser'] );
+    my $result = $self->p->process( $req, ( $mail ? ( useMail => 1 ) : () ) );
+    if ($result) {
+        $self->logger->error( "Error while looking up user: $result ("
+              . portalConsts->{$result}
+              . ")" );
+        return $self->p->sendError( $req, "User not found", 400 );
+    }
+    $result =
+      $self->p->_passwordDB->setNewPassword( $req, $password, $mail ? 1 : 0 );
+    $req->{user} = undef;
+
+    if ( $result == PE_PASSWORD_OK or $result == PE_OK ) {
+        return $self->p->sendJSONresponse( $req, { 'result' => JSON::true } );
+    }
+    else {
+        $self->logger->error( "Error while changing user password: $result ("
+              . portalConsts->{$result}
+              . ")" );
+        return $self->p->sendJSONresponse( $req, { 'result' => JSON::false } );
+    }
+
+}
+
+sub pwdConfirm {
+    my ( $self, $req ) = @_;
+
+    $self->logger->debug("Entering REST pwdConfirm method");
+
+    my $jsonBody = eval { from_json( $req->content ) };
+    if ($@) {
+        $self->logger->error("Received invalid JSON $@");
+        return $self->p->sendError( $req, "Invalid JSON", 400 );
+    }
+
+    my $user     = $jsonBody->{user};
+    my $password = $jsonBody->{password};
+
+    unless ( $user and $password ) {
+        $self->logger->error("Missing required arguments");
+        return $self->p->sendError( $req, "Missing arguments user or password",
+            400 );
+    }
+
+    $req->parameters->{user}     = $user;
+    $req->parameters->{password} = $password;
+    $req->data->{_pwdCheck}      = 1;
+    $req->data->{skipToken}      = 1;
+
+    if ( $self->p->_userDB ) {
+        $req->steps( [ $self->p->authProcess ] );
+        my $result = $self->p->process($req);
+        if ( $result == PE_PASSWORD_OK or $result == PE_OK ) {
+            return $self->p->sendJSONresponse( $req,
+                { 'result' => JSON::true } );
+        }
+        else {
+            $self->logger->error(
+                "Process returned $result (" . portalConsts->{$result} . ")" );
+            return $self->p->sendJSONresponse( $req,
+                { 'result' => JSON::false } );
+        }
+    }
+}
+
+sub getUser {
+    my ( $self, $req ) = @_;
+
+    $self->logger->debug("Entering REST getUser method");
+
+    my $jsonBody = eval { from_json( $req->content ) };
+    if ($@) {
+        $self->logger->error("Received invalid JSON $@");
+        return $self->p->sendError( $req, "Invalid JSON", 400 );
+    }
+
+    my $user = $jsonBody->{user};
+    my $mail = $jsonBody->{mail};
+
+    unless ( $user or $mail ) {
+        $self->logger->error("Missing user or mail argument");
+        return $self->p->sendError( $req, "Missing user or mail argument",
+            400 );
+    }
+
+    $req->user( $user || $mail );
+    $req->data->{_pwdCheck} = 1;
+
+    # Search user in database
+    $req->steps( [
+            'getUser',                 'setSessionInfo',
+            $self->p->groupsAndMacros, 'setLocalGroups'
+        ]
+    );
+    my $error = $self->p->process( $req, ( $mail ? ( useMail => 1 ) : () ) );
+    if ( $error == PE_OK ) {
+        return $self->p->sendJSONresponse(
+            $req,
+            {
+                'result' => JSON::true,
+                'info'   => $req->sessionInfo,
+            }
+        );
+    }
+    else {
+        return $self->p->sendJSONresponse( $req, { 'result' => JSON::false } );
+    }
+}
+
+sub myApplications {
+    my ( $self, $req ) = @_;
+    my @appslist = map {
+        my @apps = map {
+            {
+                $_->{appname} => {
+                    AppUri  => $_->{appuri},
+                    AppDesc => $_->{appdesc}
+                }
+            }
+        } @{ $_->{applications} };
+        { Category => $_->{catname}, Applications => \@apps },
+    } @{ $self->p->menu->appslist($req) };
 
     return $self->p->sendJSONresponse( $req,
-        { newtoken => $token, newimage => $image } );
+        { result => 1, myapplications => \@appslist } );
+}
+
+sub _checkSecret {
+    my ( $self, $secret ) = @_;
+    my $isValid = 0;
+
+    if ($secret) {
+        my $t;
+        if ( $t = $self->conf->{cipher}->decrypt($secret) ) {
+            if (    $t <= time + $self->conf->{restClockTolerance}
+                and $t > time - $self->conf->{restClockTolerance} )
+            {
+                $isValid = 1;
+            }
+            else {
+                $self->logger->error( 'Clock drift between servers is'
+                      . ' beyond tolerance, force denied.' );
+            }
+        }
+        else {
+            $self->logger->error('Bad key, force denied');
+        }
+    }
+
+    return $isValid;
 }
 
 1;
