@@ -6,7 +6,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_MISSINGREQATTR);
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '2.0.11';
+our $VERSION = '2.0.12';
 
 has vars => (
     is      => 'rw',
@@ -30,13 +30,13 @@ sub init {
 sub getUser {
 
     # All is done by Auth::Facebook
-    PE_OK;
+    return PE_OK;
 }
 
 sub findUser {
 
     # Nothing to do here
-    PE_OK;
+    return PE_OK;
 }
 
 sub setSessionInfo {
@@ -56,11 +56,12 @@ sub setSessionInfo {
             return PE_MISSINGREQATTR;
         }
     }
-    PE_OK;
+
+    return PE_OK;
 }
 
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
