@@ -6,7 +6,7 @@ use Lemonldap::NG::Portal::Main::Constants 'PE_OK';
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.12';
 
 # INITIALIZATION
 
@@ -26,7 +26,7 @@ sub init {
 # RUNNING METHODS
 
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
