@@ -56,11 +56,14 @@ C<LLNG::Manager::Test::_post()> call I<(see below)>.
 use strict;
 use Data::Dumper;
 use File::Find;
+use JSON;
+use XML::LibXML;
 use LWP::UserAgent;
 use Time::Fake;
 use URI::Escape;
 use MIME::Base64;
 use Lemonldap::NG::Common::FormEncode;
+use Lemonldap::NG::Common::Util qw/getPSessionID/;
 
 #use 5.10.0;
 
@@ -176,7 +179,8 @@ sub count_sessions {
 
 sub getCache {
     require Cache::FileCache;
-    return Cache::FileCache->new( {
+    return Cache::FileCache->new(
+        {
             namespace   => 'lemonldap-ng-session',
             cache_root  => $tmpDir,
             cache_depth => 0,
@@ -184,6 +188,42 @@ sub getCache {
     );
 }
 
+sub getSession {
+    my $id           = shift;
+    my @sessionsOpts = (
+        storageModule        => "Apache::Session::File",
+        storageModuleOptions => {
+            Directory     => "$tmpDir",
+            LockDirectory => "$tmpDir/lock",
+        },
+        kind => 'SSO'
+    );
+
+    return Lemonldap::NG::Common::Session->new(
+        {
+            @sessionsOpts, id => $id,
+        }
+    );
+}
+
+sub getPSession {
+    my $uid          = shift;
+    my @sessionsOpts = (
+        storageModule        => "Apache::Session::File",
+        storageModuleOptions => {
+            Directory     => "$tmpDir",
+            LockDirectory => "$tmpDir/lock",
+        },
+        kind => 'Persistent'
+    );
+
+    return Lemonldap::NG::Common::Session->new(
+        {
+            @sessionsOpts, id => getPSessionID($uid),
+        }
+    );
+}
+
 =head4 expectRedirection( $res, $location )
 
 Verify that request result is a redirection to $location. $location can be:
@@ -202,6 +242,7 @@ matching strings are returned. Example:
 =cut
 
 sub expectRedirection {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $location ) = @_;
     ok( $res->[0] == 302, ' Get redirection' )
       or explain( $res->[0], 302 );
@@ -230,6 +271,8 @@ TODO: verify javascript
 =cut
 
 sub expectAutoPost {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
     my @r      = expectForm(@_);
     my $method = pop @r;
     ok( $method =~ /^post$/i, ' Method is POST' ) or explain( $method, 'POST' );
@@ -261,13 +304,14 @@ in $query
 =cut
 
 sub expectForm {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $hostRe, $uriRe, @requiredFields ) = @_;
     expectOK($res);
     count(1);
     if (
         ok(
             $res->[2]->[0] =~
-m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
+m@<form.+?action="(?:(?:https?://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
             ' Page contains a form'
         )
       )
@@ -309,6 +353,13 @@ m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
               m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
             %fields
         );
+
+        # Add textarea
+        %fields = (
+            $res->[2]->[0] =~
+              m#<textarea.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
+            %fields
+        );
         my $query = buildForm( \%fields );
         foreach my $f (@requiredFields) {
             ok( exists $fields{$f}, qq{ Field "$f" is defined} );
@@ -329,13 +380,48 @@ Verify that result has a C<Lm-Remote-User> header and value is $user
 =cut
 
 sub expectAuthenticatedAs {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $user ) = @_;
-    ok( getHeader( $res, 'Lm-Remote-User' ) eq $user,
-        " Authenticated as $user" )
-      or explain( $res->[1], "Lm-Remote-User => $user" );
+    is( getHeader( $res, 'Lm-Remote-User' ), $user, "Authenticated as $user" );
     count(1);
 }
 
+=head4 expectSessionAttributes($app,$id,%attributes)
+
+Verify that the session contains attributes with these values
+
+=cut
+
+sub expectSessionAttributes {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $app, $id, %attributes ) = @_;
+
+    my $res = getSessionAttributes( $app, $id );
+
+    for my $attr ( keys %attributes ) {
+        is( $res->{$attr}, $attributes{$attr},
+            "Session has correct value for $attr" );
+        count(1);
+    }
+}
+
+sub getSessionAttributes {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $app, $id ) = @_;
+    my $res;
+    ok(
+        $res = $app->_get("/sessions/global/$id"),
+        "Get session using restSessionServer"
+    );
+    count(1);
+    expectOK($res);
+    ok( $res = eval { from_json( $res->[2]->[0] ) },
+        "Deserialize session content" );
+    count(1);
+
+    return $res;
+}
+
 =head4 expectOK($res)
 
 Verify that returned code is 200
@@ -343,6 +429,7 @@ Verify that returned code is 200
 =cut
 
 sub expectOK {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ($res) = @_;
     ok( $res->[0] == 200, ' HTTP code is 200' ) or explain( $res, 200 );
     count(1);
@@ -355,6 +442,7 @@ Verify that the HTTP response contains valid JSON and returns the corresponding
 =cut
 
 sub expectJSON {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ($res) = @_;
     is( $res->[0], 200, ' HTTP code is 200' ) or explain( $res, 200 );
     my %hdr = @{ $res->[1] };
@@ -368,6 +456,19 @@ sub expectJSON {
     return $json;
 }
 
+=head4 expectForbidden($res)
+
+Verify that returned code is 403.
+
+=cut
+
+sub expectForbidden {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ($res) = @_;
+    ok( $res->[0] == 403, ' HTTP code is 403' ) or explain( $res->[0], 403 );
+    count(1);
+}
+
 =head4 expectBadRequest($res)
 
 Verify that returned code is 400. Note that it works only for Ajax request
@@ -376,6 +477,7 @@ Verify that returned code is 400. Note that it works only for Ajax request
 =cut
 
 sub expectBadRequest {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ($res) = @_;
     ok( $res->[0] == 400, ' HTTP code is 400' ) or explain( $res->[0], 400 );
     count(1);
@@ -387,10 +489,14 @@ Verify that an error is displayed on the portal
 =cut
 
 sub expectPortalError {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $errnum, $message ) = @_;
-    $errnum ||= 9;
-    like( $res->[2]->[0], qr/<span trmsg="$errnum">/, $message );
-    count(1);
+    $errnum  ||= 9;
+    $message ||= "Expected portal error code";
+    my ($error) = $res->[2]->[0] =~ qr/<span trmsg="(\d+)">/;
+    ok( $error, "$message: code found on page" ) or explain $res->[2]->[0];
+    is( $error, $errnum, $message );
+    count(2);
 }
 
 =head4 expectReject( $res, $status, $code )
@@ -401,6 +507,7 @@ C<error:"$code">.  Note that it works only for Ajax request (see below).
 =cut
 
 sub expectReject {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $status, $code ) = @_;
     $status ||= 401;
     cmp_ok( $res->[0], '==', $status, " Response status is $status" );
@@ -424,6 +531,7 @@ its value.
 =cut
 
 sub expectCookie {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $cookieName ) = @_;
     $cookieName ||= 'lemonldap';
     my $cookies = getCookies($res);
@@ -436,6 +544,24 @@ sub expectCookie {
     return $id;
 }
 
+=head4 expectPdata( $res );
+
+Check if the pdata cookie exists and returns its deserialized value.
+
+=cut
+
+sub expectPdata {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ($res) = @_;
+    my $val = expectCookie( $res, "lemonldappdata" );
+    ok( $val, "Pdata is not empty" );
+    count(1);
+    my $pdata;
+    eval { $pdata = JSON::from_json( uri_unescape($val) ); };
+    diag($@) if $@;
+    return $pdata;
+}
+
 =head4 exceptCspFormOK( $res, $host )
 
 Verify that C<Content-Security-Policy> header allows one to connect to $host.
@@ -443,6 +569,7 @@ Verify that C<Content-Security-Policy> header allows one to connect to $host.
 =cut
 
 sub exceptCspFormOK {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $res, $host ) = @_;
     return 1 unless ($host);
     my $csp = getHeader( $res, 'Content-Security-Policy' );
@@ -464,6 +591,28 @@ sub exceptCspFormOK {
     count(1);
 }
 
+sub expectCspChildOK {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $res, $host ) = @_;
+    return 1 unless ($host);
+    my $csp = getHeader( $res, 'Content-Security-Policy' );
+    ok( $csp, "Content-Security-Policy header found" );
+    count(1);
+    like( $csp, qr/child-src[^;]*\Q$host\E/, "Found $host in CSP child-src" );
+    count(1);
+}
+
+sub getHtmlElement {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $res, $xpath ) = @_;
+
+    # Use recover to ignore parsing errors
+    my $doc =
+      XML::LibXML->new->load_html( string => $res->[2]->[0], recover => 2 );
+
+    return $doc->findnodes($xpath);
+}
+
 =head4 getCookies($res)
 
 Returns an hash ref with names => values of cookies set by server.
@@ -545,6 +694,7 @@ Registers a new LLNG instance
 =cut
 
 sub register {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $type, $constructor ) = @_;
     my $obj;
     @Lemonldap::NG::Handler::Main::_onReload = ();
@@ -578,7 +728,7 @@ Encode URL like the handler would, see ::Handler::Main
 
 sub encodeUrl {
     my ($url) = @_;
-    return uri_escape( encode_base64( $url, '' ) );
+    return encode_base64( $url, '' );
 }
 
 =head2 LLNG::Manager::Test Class
@@ -589,9 +739,24 @@ package LLNG::Manager::Test;
 
 use strict;
 use Mouse;
+use IO::String;
 
 extends 'Lemonldap::NG::Common::PSGI::Cli::Lib';
 
+# try to find template dir in @INC
+my $templateDir = "site/templates";
+unless ( -d $templateDir ) {
+    for (@INC) {
+        if ( -d "$_/site/templates" ) {
+            $templateDir = "$_/site/templates";
+            last;
+        }
+    }
+    unless ( -d $templateDir ) {
+        die "Could not find template dir";
+    }
+}
+
 our $defaultIni = {
     configStorage => {
         type    => 'File',
@@ -606,7 +771,7 @@ our $defaultIni = {
     logLevel              => 'error',
     cookieName            => 'lemonldap',
     domain                => 'example.com',
-    templateDir           => 'site/templates',
+    templateDir           => $templateDir,
     staticPrefix          => '/static',
     tokenUseGlobalStorage => 0,
     securedCookie         => 0,
@@ -620,26 +785,26 @@ our $defaultIni = {
     ),
     https                => 0,
     globalStorageOptions => {
-        Directory     => $tmpDir,
-        LockDirectory => "$tmpDir/lock",
+        Directory      => $tmpDir,
+        LockDirectory  => "$tmpDir/lock",
         generateModule =>
           'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
     },
     casStorageOptions => {
-        Directory     => "$tmpDir/saml",
-        LockDirectory => "$tmpDir/saml/lock",
+        Directory      => "$tmpDir/saml",
+        LockDirectory  => "$tmpDir/saml/lock",
         generateModule =>
           'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
     },
     samlStorageOptions => {
-        Directory     => "$tmpDir/saml",
-        LockDirectory => "$tmpDir/saml/lock",
+        Directory      => "$tmpDir/saml",
+        LockDirectory  => "$tmpDir/saml/lock",
         generateModule =>
           'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
     },
     oidcStorageOptions => {
-        Directory     => "$tmpDir/saml",
-        LockDirectory => "$tmpDir/saml/lock",
+        Directory      => "$tmpDir/saml",
+        LockDirectory  => "$tmpDir/saml/lock",
         generateModule =>
           'Lemonldap::NG::Common::Apache::Session::Generate::SHA256',
     },
@@ -674,6 +839,7 @@ has p => ( is => 'rw' );
 
 =cut
 
+has accept => ( is => 'rw', default => 'application/json, text/plain, */*' );
 has confFailure => ( is => 'rw' );
 
 has ini => (
@@ -737,6 +903,36 @@ has ini => (
 
 =head3 Methods
 
+=head4 login($client, $uid, $getParams)
+Do a simple login using the Demo backend
+=cut
+
+sub login {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $self, $uid, $getParams ) = @_;
+    my $res;
+    $getParams ||= {};
+
+    my $query = main::buildForm(
+        {
+            user     => $uid,
+            password => $uid,
+            %$getParams,
+        }
+    );
+    main::ok(
+        $res = $self->_post(
+            '/',
+            IO::String->new($query),
+            length => length($query),
+        ),
+        'Auth query'
+    );
+    main::expectOK($res);
+    my $id = main::expectCookie($res);
+    return $id;
+}
+
 =head4 logout($id)
 
 Launch a C</?logout=1> request an test:
@@ -745,7 +941,7 @@ Launch a C</?logout=1> request an test:
 
 =item if response is 200
 
-=item if cookie 'lemonldap' and 'lemonldappdata' have no value
+=item if cookie $cookieName and "${cookieName}pdata" have no value
 
 =item if a GET request with previous cookie value I<($i)> is rejected
 
@@ -754,13 +950,16 @@ Launch a C</?logout=1> request an test:
 =cut
 
 sub logout {
-    my ( $self, $id ) = @_;
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ( $self, $id, $cookieName ) = @_;
     my $res;
+    $cookieName ||= 'lemonldap';
+
     main::ok(
         $res = $self->_get(
             '/',
             query  => 'logout',
-            cookie => "lemonldap=$id",
+            cookie => "$cookieName=$id",
             accept => 'text/html'
         ),
         'Logout request'
@@ -769,11 +968,12 @@ sub logout {
       or main::explain( $res->[0], 200 );
     my $c;
     main::ok(
-        ( defined( $c = main::getCookies($res)->{lemonldap} ) and not $c ),
+        ( defined( $c = main::getCookies($res)->{$cookieName} ) and not $c ),
         ' Cookie is deleted' )
       or main::explain( $res->[1], "Set-Cookie => 'lemonldap='" );
-    main::ok( not( main::getCookies($res)->{lemonldappdata} ), ' No pdata' );
-    main::ok( $res = $self->_get( '/', cookie => "lemonldap=$id" ),
+    main::ok( not( main::getCookies($res)->{"${cookieName}pdata"} ),
+        ' No pdata' );
+    main::ok( $res = $self->_get( '/', cookie => "$cookieName=$id" ),
         'Disconnect request' )
       or explain( $res, '[<code>,<hdrs>,<content>]' );
     main::ok( $res->[0] == 401, ' Response is 401' )
@@ -812,9 +1012,15 @@ to test content I<(to launch a C<expectForm()> for example)>.
 
 sub _get {
     my ( $self, $path, %args ) = @_;
-    my $res = $self->app->( {
-            'HTTP_ACCEPT' => $args{accept}
-              || 'application/json, text/plain, */*',
+
+    # Automatically serialize query if given as HASHREF
+    if ( ref( $args{query} ) eq "HASH" ) {
+        $args{query} = main::buildForm( $args{query} );
+    }
+
+    my $res = $self->app->(
+        {
+            'HTTP_ACCEPT'          => $args{accept} // $self->accept,
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
@@ -863,11 +1069,25 @@ Example with IO::String:
 
 sub _post {
     my ( $self, $path, $body, %args ) = @_;
+
+    # If $body is a HASHREF, serialize as string
+    if ( ref($body) eq "HASH" ) {
+        $body = main::buildForm($body);
+    }
+
+    # If $body is a string, wrap in IO::Handle
+    unless ( ref($body) ) {
+        $args{length} = length($body);
+
+        # We must force a string copy here to avoid circular references
+        $body = IO::String->new("$body");
+    }
+
     die "$body must be a IO::Handle"
       unless ( ref($body) and $body->can('read') );
-    my $res = $self->app->( {
-            'HTTP_ACCEPT' => $args{accept}
-              || 'application/json, text/plain, */*',
+    my $res = $self->app->(
+        {
+            'HTTP_ACCEPT'          => $args{accept} // $self->accept,
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
@@ -895,8 +1115,8 @@ sub _post {
             ( $args{custom} ? %{ $args{custom} } : () ),
             'psgix.input.buffered' => 0,
             'psgi.input'           => $body,
-            'CONTENT_LENGTH' => $args{length} // scalar( ( stat $body )[7] ),
-            'CONTENT_TYPE'   => $args{type}
+            'CONTENT_LENGTH'       => $args{length},
+            'CONTENT_TYPE'         => $args{type}
               || 'application/x-www-form-urlencoded',
         }
     );
